@@ -1,7 +1,7 @@
 /*
  *  Entangle: Tethered Camera Control & Capture
  *
- *  Copyright (C) 2009-2017 Daniel P. Berrange
+ *  Copyright (C) 2009-2018 Daniel P. Berrange
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ struct _EntangleCameraAutomataPrivate {
     gulong sigFileDownload;
 };
 
-G_DEFINE_TYPE(EntangleCameraAutomata, entangle_camera_automata, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE(EntangleCameraAutomata, entangle_camera_automata, G_TYPE_OBJECT);
 
 enum {
     PROP_0,
@@ -174,8 +174,6 @@ static void entangle_camera_automata_class_init(EntangleCameraAutomataClass *kla
                  g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE,
                  0);
-
-    g_type_class_add_private(klass, sizeof(EntangleCameraAutomataPrivate));
 }
 
 
