@@ -1,7 +1,7 @@
 /*
  *  Entangle: Tethered Camera Control & Capture
  *
- *  Copyright (C) 2009-2017 Daniel P. Berrange
+ *  Copyright (C) 2009-2018 Daniel P. Berrange
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -348,6 +348,7 @@ static GdkPixbuf *entangle_pixbuf_open_image_preview_raw(EntangleImage *image,
     result = gdk_pixbuf_loader_get_pixbuf(loader);
 
     if (result) {
+        g_object_ref(result);
         if (applyOrientation) {
             GdkPixbuf *tmp = entangle_pixbuf_auto_rotate(result, metadata);
             g_object_unref(result);
@@ -371,6 +372,7 @@ static GdkPixbuf *entangle_pixbuf_open_image_preview_raw(EntangleImage *image,
     libraw_close(raw);
 
     gdk_pixbuf_loader_close(loader, NULL);
+    g_object_unref(loader);
 
     return result;
 }
