@@ -1,7 +1,7 @@
 /*
  *  Entangle: Tethered Camera Control & Capture
  *
- *  Copyright (C) 2009-2017 Daniel P. Berrange
+ *  Copyright (C) 2009-2018 Daniel P. Berrange
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -65,7 +65,7 @@ struct _EntangleApplicationPrivate {
     PeasExtensionSet *pluginExt;
 };
 
-G_DEFINE_TYPE(EntangleApplication, entangle_application, GTK_TYPE_APPLICATION);
+G_DEFINE_TYPE_WITH_PRIVATE(EntangleApplication, entangle_application, GTK_TYPE_APPLICATION);
 
 enum {
     PROP_0,
@@ -276,17 +276,10 @@ static void do_action_manual(GSimpleAction *act G_GNUC_UNUSED,
     g_return_if_fail(ENTANGLE_IS_APPLICATION(opaque));
 
     EntangleApplication *app = ENTANGLE_APPLICATION(opaque);
-#if GTK_CHECK_VERSION(3, 22, 0)
     gtk_show_uri_on_window(gtk_application_get_active_window(GTK_APPLICATION(app)),
                            "help:entangle",
                            GDK_CURRENT_TIME,
                            NULL);
-#else
-    gtk_show_uri(gtk_widget_get_screen(GTK_WIDGET(src)),
-                 "help:entangle",
-                 GDK_CURRENT_TIME,
-                 NULL);
-#endif
 }
 
 static void do_action_new_window(GSimpleAction *act G_GNUC_UNUSED,
@@ -341,20 +334,10 @@ static void entangle_application_startup(GApplication *gapp)
     EntangleApplicationPrivate *priv = app->priv;
     GList *cameras = NULL, *tmp = NULL;
     gboolean gotcamera = FALSE;
-    GtkBuilder *builder;
-    GMenuModel *appmenu;
     gsize i;
 
     (*G_APPLICATION_CLASS(entangle_application_parent_class)->startup)(gapp);
 
-    builder = gtk_builder_new_from_resource("/org/entangle_photo/Manager/frontend/entangle-application-menus.ui");
-
-    appmenu = G_MENU_MODEL(gtk_builder_get_object(builder, "app-menu"));
-
-    gtk_application_set_app_menu(GTK_APPLICATION(gapp), appmenu);
-
-    g_object_unref(builder);
-
     g_action_map_add_action_entries(G_ACTION_MAP(gapp),
                                     actions, G_N_ELEMENTS(actions),
                                     app);
@@ -437,8 +420,6 @@ static void entangle_application_class_init(EntangleApplicationClass *klass)
                                                         G_PARAM_STATIC_NAME |
                                                         G_PARAM_STATIC_NICK |
                                                         G_PARAM_STATIC_BLURB));
-
-    g_type_class_add_private(klass, sizeof(EntangleApplicationPrivate));
 }
 
 
