@@ -1,7 +1,7 @@
 /*
  *  Entangle: Tethered Camera Control & Capture
  *
- *  Copyright (C) 2009-2017 Daniel P. Berrange
+ *  Copyright (C) 2009-2018 Daniel P. Berrange
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -401,7 +401,7 @@ static EntangleCamera *entangle_picker_get_selected_camera(EntangleCameraPicker
             return NULL;
 
         gchar *port = g_strdup_printf("ptpip:%s", addr);
-        EntangleCamera *cam = entangle_camera_new("PTP/IP Camera", port, FALSE, FALSE, FALSE);
+        EntangleCamera *cam = entangle_camera_new("PTP/IP Camera", port, NULL, FALSE, FALSE, FALSE);
         g_free(port);
 
         return cam;
