@@ -1,7 +1,7 @@
 /*
  *  Entangle: Tethered Camera Control & Capture
  *
- *  Copyright (C) 2009-2017 Daniel P. Berrange
+ *  Copyright (C) 2009-2018 Daniel P. Berrange
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,6 +32,10 @@
 #include "entangle-control-text.h"
 #include "entangle-control-toggle.h"
 
+#ifdef GDK_WINDOWING_WAYLAND
+#include <gdk/gdkwayland.h>
+#endif
+
 #define ENTANGLE_CONTROL_PANEL_GET_PRIVATE(obj)                         \
     (G_TYPE_INSTANCE_GET_PRIVATE((obj), ENTANGLE_TYPE_CONTROL_PANEL, EntangleControlPanelPrivate))
 
@@ -47,7 +51,7 @@ struct _EntangleControlPanelPrivate {
     gsize rows;
 };
 
-G_DEFINE_TYPE(EntangleControlPanel, entangle_control_panel, GTK_TYPE_EXPANDER);
+G_DEFINE_TYPE_WITH_PRIVATE(EntangleControlPanel, entangle_control_panel, GTK_TYPE_EXPANDER);
 
 enum {
     PROP_O,
@@ -434,11 +438,20 @@ static void do_setup_control(EntangleControlPanel *panel,
                                entangle_control_choice_entry_get(ENTANGLE_CONTROL_CHOICE(control), n),
                                -1);
         }
+        g_free(text);
 
         if (entangle_control_get_readonly(control))
             gtk_widget_set_sensitive(value, FALSE);
         gtk_combo_box_set_active(GTK_COMBO_BOX(value), active);
 
+#ifdef GDK_WINDOWING_WAYLAND
+        /* Hack until https://gitlab.gnome.org/GNOME/gtk/issues/1463 is fixed */
+        if (GDK_IS_WAYLAND_DISPLAY(gtk_widget_get_display(GTK_WIDGET(value))) &&
+            (entangle_control_choice_entry_count(ENTANGLE_CONTROL_CHOICE(control)) > 15)) {
+            gtk_combo_box_set_wrap_width(GTK_COMBO_BOX(value), 1);
+        }
+#endif
+
         g_signal_connect(value, "changed",
                          G_CALLBACK(do_update_control_combo), panel);
         g_signal_connect(control, "notify::value",
@@ -814,12 +827,7 @@ static void do_setup_controls(EntangleControlPanel *panel)
                               menu);
     gtk_widget_set_hexpand(settingsButton, TRUE);
     gtk_widget_set_halign(settingsButton, GTK_ALIGN_END);
-#if GTK_CHECK_VERSION(3, 12, 0)
     gtk_widget_set_margin_end(settingsButton, 6);
-#else
-    gtk_widget_set_margin_right(settingsButton, 6);
-#endif
-
 
     gtk_grid_attach(GTK_GRID(priv->grid), settingsButton, 1, priv->rows, 2, 1);
 
@@ -833,6 +841,7 @@ static void do_setup_camera(EntangleControlPanel *panel)
     g_return_if_fail(ENTANGLE_IS_CONTROL_PANEL(panel));
 
     EntangleControlPanelPrivate *priv = panel->priv;
+    EntangleControlGroup *ctrls = NULL;
 
     gtk_container_foreach(GTK_CONTAINER(priv->grid), do_control_remove, panel);
     priv->rows = 0;
@@ -844,7 +853,7 @@ static void do_setup_camera(EntangleControlPanel *panel)
         gtk_widget_set_halign(label, GTK_ALIGN_FILL);
         gtk_grid_attach(GTK_GRID(priv->grid), label, 0, 0, 2, 1);
         gtk_widget_show_all(GTK_WIDGET(panel));
-    } else if (entangle_camera_get_controls(priv->camera, NULL) == NULL) {
+    } else if ((ctrls = entangle_camera_get_controls(priv->camera, NULL)) == NULL) {
         GtkWidget *label = gtk_label_new(_("No controls available"));
         gtk_widget_set_hexpand(label, TRUE);
         gtk_widget_set_halign(label, GTK_ALIGN_FILL);
@@ -853,6 +862,8 @@ static void do_setup_camera(EntangleControlPanel *panel)
     } else {
         do_setup_controls(panel);
     }
+    if (ctrls)
+        g_object_unref(ctrls);
 }
 
 
@@ -987,8 +998,6 @@ static void entangle_control_panel_class_init(EntangleControlPanelClass *klass)
                                                          G_PARAM_STATIC_NAME |
                                                          G_PARAM_STATIC_NICK |
                                                          G_PARAM_STATIC_BLURB));
-
-    g_type_class_add_private(klass, sizeof(EntangleControlPanelPrivate));
 }
 
 
