@@ -1,7 +1,7 @@
 /*
  *  Entangle: Tethered Camera Control & Capture
  *
- *  Copyright (C) 2009-2017 Daniel P. Berrange
+ *  Copyright (C) 2009-2018 Daniel P. Berrange
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,12 +40,13 @@ struct _EntangleImageDisplayPrivate {
     gdouble aspectRatio;
     gdouble maskOpacity;
     gboolean maskEnabled;
+    gboolean overexposureHighlighting;
 
     gboolean focusPoint;
     EntangleImageDisplayGrid gridDisplay;
 };
 
-G_DEFINE_TYPE(EntangleImageDisplay, entangle_image_display, GTK_TYPE_DRAWING_AREA);
+G_DEFINE_TYPE_WITH_PRIVATE(EntangleImageDisplay, entangle_image_display, GTK_TYPE_DRAWING_AREA);
 
 enum {
     PROP_O,
@@ -57,6 +58,7 @@ enum {
     PROP_MASK_ENABLED,
     PROP_FOCUS_POINT,
     PROP_GRID_DISPLAY,
+    PROP_OVEREXPOSURE_HIGHLIGHTING,
 };
 
 
@@ -115,6 +117,29 @@ static void do_entangle_image_display_render_pixmap(EntangleImageDisplay *displa
 
         tmp = tmp->next;
     }
+
+    if (priv->overexposureHighlighting) {
+        /* Check for blown highlights (overexposure), and set such pixels
+           to red. */
+        cairo_surface_flush(priv->pixmap);
+        unsigned char *data = cairo_image_surface_get_data(priv->pixmap);
+        gboolean blown = FALSE;
+        unsigned char *p = data;
+        for (int i=0; i < height; i++) {
+            for (int j=0; j < width; j++, p+=4) {
+                if (p[0] == 255 || p[1] == 255 || p[2] == 255) {
+                    p[0] = 0;
+                    p[1] = 0;
+                    p[2] = 254;
+                    blown = TRUE;
+                }
+            }
+        }
+
+        if (blown)
+            cairo_surface_mark_dirty(priv->pixmap);
+    }
+
     cairo_destroy(cr);
 }
 
@@ -221,6 +246,10 @@ static void entangle_image_display_get_property(GObject *object,
             g_value_set_enum(value, priv->gridDisplay);
             break;
 
+        case PROP_OVEREXPOSURE_HIGHLIGHTING:
+            g_value_set_boolean(value, priv->overexposureHighlighting);
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         }
@@ -266,6 +295,10 @@ static void entangle_image_display_set_property(GObject *object,
             entangle_image_display_set_grid_display(display, g_value_get_enum(value));
             break;
 
+        case PROP_OVEREXPOSURE_HIGHLIGHTING:
+            entangle_image_display_set_overexposure_highlighting(display, g_value_get_boolean(value));
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         }
@@ -784,7 +817,16 @@ static void entangle_image_display_class_init(EntangleImageDisplayClass *klass)
                                                       G_PARAM_STATIC_NICK |
                                                       G_PARAM_STATIC_BLURB));
 
-    g_type_class_add_private(klass, sizeof(EntangleImageDisplayPrivate));
+    g_object_class_install_property(object_class,
+                                    PROP_OVEREXPOSURE_HIGHLIGHTING,
+                                    g_param_spec_boolean("overexposure-highlighting",
+                                                         "Overexposure highlighting",
+                                                         "Highlight areas of overexposure",
+                                                         TRUE,
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
 }
 
 
@@ -804,6 +846,7 @@ static void entangle_image_display_init(EntangleImageDisplay *display)
     priv->maskOpacity = 0.9;
     priv->aspectRatio = 1.33;
     priv->maskEnabled = FALSE;
+    priv->overexposureHighlighting = TRUE;
 }
 
 
@@ -978,7 +1021,7 @@ void entangle_image_display_set_aspect_ratio(EntangleImageDisplay *display,
     priv->aspectRatio = aspect;
 
     if (gtk_widget_get_visible((GtkWidget*)display))
-        gtk_widget_queue_resize(GTK_WIDGET(display));
+        gtk_widget_queue_draw(GTK_WIDGET(display));
 }
 
 
@@ -1002,7 +1045,7 @@ void entangle_image_display_set_mask_opacity(EntangleImageDisplay *display,
     priv->maskOpacity = opacity;
 
     if (gtk_widget_get_visible((GtkWidget*)display))
-        gtk_widget_queue_resize(GTK_WIDGET(display));
+        gtk_widget_queue_draw(GTK_WIDGET(display));
 }
 
 
@@ -1026,7 +1069,7 @@ void entangle_image_display_set_mask_enabled(EntangleImageDisplay *display,
     priv->maskEnabled = enabled;
 
     if (gtk_widget_get_visible((GtkWidget*)display))
-        gtk_widget_queue_resize(GTK_WIDGET(display));
+        gtk_widget_queue_draw(GTK_WIDGET(display));
 }
 
 
@@ -1049,7 +1092,7 @@ void entangle_image_display_set_focus_point(EntangleImageDisplay *display,
     priv->focusPoint = enabled;
 
     if (gtk_widget_get_visible((GtkWidget*)display))
-        gtk_widget_queue_resize(GTK_WIDGET(display));
+        gtk_widget_queue_draw(GTK_WIDGET(display));
 }
 
 
@@ -1068,7 +1111,7 @@ void entangle_image_display_set_grid_display(EntangleImageDisplay *display,
     priv->gridDisplay = mode;
 
     if (gtk_widget_get_visible((GtkWidget*)display))
-        gtk_widget_queue_resize(GTK_WIDGET(display));
+        gtk_widget_queue_draw(GTK_WIDGET(display));
 }
 
 
@@ -1080,6 +1123,32 @@ EntangleImageDisplayGrid entangle_image_display_get_grid_display(EntangleImageDi
 }
 
 
+void entangle_image_display_set_overexposure_highlighting(EntangleImageDisplay *display,
+                                                          gboolean enabled)
+{
+    g_return_if_fail(ENTANGLE_IS_IMAGE_DISPLAY(display));
+
+    EntangleImageDisplayPrivate *priv = display->priv;
+
+    priv->overexposureHighlighting = enabled;
+
+    entangle_image_display_try_render_pixmap(display);
+    if (gtk_widget_get_visible((GtkWidget*)display))
+        gtk_widget_queue_draw(GTK_WIDGET(display));
+}
+
+
+gboolean entangle_image_display_get_overexposure_highlighting(EntangleImageDisplay *display)
+{
+    g_return_val_if_fail(ENTANGLE_IS_IMAGE_DISPLAY(display), FALSE);
+
+    EntangleImageDisplayPrivate *priv = display->priv;
+
+    return priv->overexposureHighlighting;
+}
+
+
+
 gboolean entangle_image_display_get_loaded(EntangleImageDisplay *display)
 {
     EntangleImage *image = entangle_image_display_get_image(display);
