@@ -1,7 +1,7 @@
 /*
  *  Entangle: Tethered Camera Control & Capture
  *
- *  Copyright (C) 2009-2017 Daniel P. Berrange
+ *  Copyright (C) 2009-2018 Daniel P. Berrange
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -87,6 +87,7 @@ struct _EntanglePreferencesDisplayPrivate {
     GtkToggleButton *interfaceHistogramLinear;
 
     GtkToggleButton *imgMaskEnabled;
+    GtkToggleButton *imgOverexposureHighlighting;
     GtkLabel *imgAspectRatioLabel;
     GtkLabel *imgMaskOpacityLabel;
     GtkComboBox *imgAspectRatio;
@@ -132,6 +133,7 @@ void do_capture_delete_file_toggled(GtkToggleButton *src, EntanglePreferencesDis
 void do_capture_sync_clock_toggled(GtkToggleButton *src, EntanglePreferencesDisplay *display);
 
 void do_img_mask_enabled_toggled(GtkToggleButton *src, EntanglePreferencesDisplay *display);
+void do_img_overexposure_highlighting_toggled(GtkToggleButton *src, EntanglePreferencesDisplay *display);
 void do_img_aspect_ratio_changed(GtkComboBox *src, EntanglePreferencesDisplay *display);
 void do_img_mask_opacity_changed(GtkSpinButton *src, EntanglePreferencesDisplay *display);
 void do_img_focus_point_toggled(GtkToggleButton *src, EntanglePreferencesDisplay *display);
@@ -298,6 +300,15 @@ static void entangle_preferences_display_notify(GObject *object,
 
         if (newvalue != oldvalue)
             gtk_toggle_button_set_active(priv->imgMaskEnabled, newvalue);
+    } else if (strcmp(spec->name, "img-overexposure-highlighting") == 0) {
+        gboolean newvalue;
+        gboolean oldvalue;
+
+        g_object_get(object, spec->name, &newvalue, NULL);
+        oldvalue = gtk_toggle_button_get_active(priv->imgOverexposureHighlighting);
+
+        if (newvalue != oldvalue)
+            gtk_toggle_button_set_active(priv->imgOverexposureHighlighting, newvalue);
     } else if (strcmp(spec->name, "img-aspect-ratio") == 0) {
         gchar *newvalue;
         const gchar *oldvalue;
@@ -418,7 +429,8 @@ static void entangle_preferences_display_refresh(EntanglePreferencesDisplay *pre
 
     EntanglePreferencesDisplayPrivate *priv = entangle_preferences_display_get_instance_private(preferences);
     EntangleColourProfile *profile;
-    const gchar *ratio;
+    gchar *ratio;
+    gchar *pattern;
     GtkAdjustment *adjust;
     gboolean hasContinuousPreview;
     gboolean hasRatio;
@@ -447,7 +459,8 @@ static void entangle_preferences_display_refresh(EntanglePreferencesDisplay *pre
     gtk_toggle_button_set_active(priv->interfaceHistogramLinear,
                                  entangle_preferences_interface_get_histogram_linear(priv->prefs));
 
-    gtk_entry_set_text(priv->captureFilenamePattern, entangle_preferences_capture_get_filename_pattern(priv->prefs));
+    pattern = entangle_preferences_capture_get_filename_pattern(priv->prefs);
+    gtk_entry_set_text(priv->captureFilenamePattern, pattern);
 
     hasContinuousPreview = entangle_preferences_capture_get_continuous_preview(priv->prefs);
     gtk_toggle_button_set_active(priv->captureContinuousPreview, hasContinuousPreview);
@@ -464,6 +477,9 @@ static void entangle_preferences_display_refresh(EntanglePreferencesDisplay *pre
 
     gtk_toggle_button_set_active(priv->imgMaskEnabled, hasRatio);
 
+    gtk_toggle_button_set_active(priv->imgOverexposureHighlighting,
+                                 entangle_preferences_img_get_overexposure_highlighting(priv->prefs));
+
     gtk_combo_box_set_active_id(priv->imgAspectRatio, ratio);
     gtk_widget_set_sensitive(GTK_WIDGET(priv->imgAspectRatio), hasRatio);
 
@@ -513,6 +529,9 @@ static void entangle_preferences_display_refresh(EntanglePreferencesDisplay *pre
     gdk_rgba_parse(&ghl, hl);
     gtk_color_chooser_set_rgba(priv->imgHighlight, &ghl);
     g_free(hl);
+
+    g_free(ratio);
+    g_free(pattern);
 }
 
 
@@ -622,6 +641,8 @@ static void entangle_preferences_display_class_init(EntanglePreferencesDisplayCl
 
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass),
                                                  EntanglePreferencesDisplay, imgMaskEnabled);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass),
+                                                 EntanglePreferencesDisplay, imgOverexposureHighlighting);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass),
                                                  EntanglePreferencesDisplay, imgAspectRatioLabel);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass),
@@ -875,6 +896,17 @@ void do_img_mask_enabled_toggled(GtkToggleButton *src, EntanglePreferencesDispla
 }
 
 
+void do_img_overexposure_highlighting_toggled(GtkToggleButton *src, EntanglePreferencesDisplay *preferences)
+{
+    g_return_if_fail(ENTANGLE_IS_PREFERENCES_DISPLAY(preferences));
+
+    EntanglePreferencesDisplayPrivate *priv = entangle_preferences_display_get_instance_private(preferences);
+    gboolean enabled = gtk_toggle_button_get_active(src);
+
+    entangle_preferences_img_set_overexposure_highlighting(priv->prefs, enabled);
+}
+
+
 void do_img_aspect_ratio_changed(GtkComboBox *src, EntanglePreferencesDisplay *preferences)
 {
     g_return_if_fail(ENTANGLE_IS_PREFERENCES_DISPLAY(preferences));
