@@ -1,7 +1,7 @@
 /*
  *  Entangle: Tethered Camera Control & Capture
  *
- *  Copyright (C) 2009-2017 Daniel P. Berrange
+ *  Copyright (C) 2009-2018 Daniel P. Berrange
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ struct _EntanglePreferencesPrivate {
     GSettings *imgSettings;
 };
 
-G_DEFINE_TYPE(EntanglePreferences, entangle_preferences, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE(EntanglePreferences, entangle_preferences, G_TYPE_OBJECT);
 
 #define SETTING_INTERFACE                  "interface"
 #define SETTING_CAPTURE                    "capture"
@@ -67,6 +67,7 @@ G_DEFINE_TYPE(EntanglePreferences, entangle_preferences, G_TYPE_OBJECT);
 #define SETTING_IMG_ASPECT_RATIO           "aspect-ratio"
 #define SETTING_IMG_MASK_OPACITY           "mask-opacity"
 #define SETTING_IMG_MASK_ENABLED           "mask-enabled"
+#define SETTING_IMG_OVEREXPOSURE_HIGHLIGHTING "overexposure-highlighting"
 #define SETTING_IMG_FOCUS_POINT            "focus-point"
 #define SETTING_IMG_GRID_LINES             "grid-lines"
 #define SETTING_IMG_EMBEDDED_PREVIEW       "embedded-preview"
@@ -96,6 +97,7 @@ G_DEFINE_TYPE(EntanglePreferences, entangle_preferences, G_TYPE_OBJECT);
 #define PROP_NAME_IMG_ASPECT_RATIO           SETTING_IMG "-" SETTING_IMG_ASPECT_RATIO
 #define PROP_NAME_IMG_MASK_OPACITY           SETTING_IMG "-" SETTING_IMG_MASK_OPACITY
 #define PROP_NAME_IMG_MASK_ENABLED           SETTING_IMG "-" SETTING_IMG_MASK_ENABLED
+#define PROP_NAME_IMG_OVEREXPOSURE_HIGHLIGHTING SETTING_IMG "-" SETTING_IMG_OVEREXPOSURE_HIGHLIGHTING
 #define PROP_NAME_IMG_FOCUS_POINT            SETTING_IMG "-" SETTING_IMG_FOCUS_POINT
 #define PROP_NAME_IMG_GRID_LINES             SETTING_IMG "-" SETTING_IMG_GRID_LINES
 #define PROP_NAME_IMG_EMBEDDED_PREVIEW       SETTING_IMG "-" SETTING_IMG_EMBEDDED_PREVIEW
@@ -127,6 +129,7 @@ enum {
     PROP_IMG_ASPECT_RATIO,
     PROP_IMG_MASK_OPACITY,
     PROP_IMG_MASK_ENABLED,
+    PROP_IMG_OVEREXPOSURE_HIGHLIGHTING,
     PROP_IMG_FOCUS_POINT,
     PROP_IMG_GRID_LINES,
     PROP_IMG_EMBEDDED_PREVIEW,
@@ -171,6 +174,7 @@ static void entangle_preferences_get_property(GObject *object,
     EntangleColourProfile *prof;
     gchar *dir;
     gchar *file;
+    gchar *val;
 
     switch (prop_id)
         {
@@ -198,6 +202,7 @@ static void entangle_preferences_get_property(GObject *object,
             if (!dir)
                 dir = entangle_find_picture_dir();
             g_value_set_string(value, dir);
+            g_free(dir);
             break;
 
         case PROP_CAPTURE_FILENAME_PATTERN:
@@ -278,10 +283,17 @@ static void entangle_preferences_get_property(GObject *object,
                                                        SETTING_IMG_MASK_ENABLED));
             break;
 
+        case PROP_IMG_OVEREXPOSURE_HIGHLIGHTING:
+            g_value_set_boolean(value,
+                                g_settings_get_boolean(priv->imgSettings,
+                                                       SETTING_IMG_OVEREXPOSURE_HIGHLIGHTING));
+            break;
+
         case PROP_IMG_ASPECT_RATIO:
-            g_value_set_string(value,
-                               g_settings_get_string(priv->imgSettings,
-                                                     SETTING_IMG_ASPECT_RATIO));
+            val = g_settings_get_string(priv->imgSettings,
+                                        SETTING_IMG_ASPECT_RATIO);
+            g_value_set_string(value, val);
+            g_free(val);
             break;
 
         case PROP_IMG_MASK_OPACITY:
@@ -321,15 +333,17 @@ static void entangle_preferences_get_property(GObject *object,
             break;
 
         case PROP_IMG_BACKGROUND:
-            g_value_set_string(value,
-                               g_settings_get_string(priv->imgSettings,
-                                                     SETTING_IMG_BACKGROUND));
+            val = g_settings_get_string(priv->imgSettings,
+                                        SETTING_IMG_BACKGROUND);
+            g_value_set_string(value, val);
+            g_free(val);
             break;
 
         case PROP_IMG_HIGHLIGHT:
-            g_value_set_string(value,
-                               g_settings_get_string(priv->imgSettings,
-                                                     SETTING_IMG_HIGHLIGHT));
+            val = g_settings_get_string(priv->imgSettings,
+                                        SETTING_IMG_HIGHLIGHT);
+            g_value_set_string(value, val);
+            g_free(val);
             break;
 
         default:
@@ -450,6 +464,12 @@ static void entangle_preferences_set_property(GObject *object,
                                    g_value_get_boolean(value));
             break;
 
+        case PROP_IMG_OVEREXPOSURE_HIGHLIGHTING:
+            g_settings_set_boolean(priv->imgSettings,
+                                   SETTING_IMG_OVEREXPOSURE_HIGHLIGHTING,
+                                   g_value_get_boolean(value));
+            break;
+
         case PROP_IMG_ASPECT_RATIO:
             g_settings_set_string(priv->imgSettings,
                                   SETTING_IMG_ASPECT_RATIO,
@@ -725,6 +745,17 @@ static void entangle_preferences_class_init(EntanglePreferencesClass *klass)
                                                          G_PARAM_STATIC_NICK |
                                                          G_PARAM_STATIC_BLURB));
 
+    g_object_class_install_property(object_class,
+                                    PROP_IMG_OVEREXPOSURE_HIGHLIGHTING,
+                                    g_param_spec_boolean(PROP_NAME_IMG_OVEREXPOSURE_HIGHLIGHTING,
+                                                         "Overexposure highlighting",
+                                                         "Highlight areas of overexposure",
+                                                         TRUE,
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
+
     g_object_class_install_property(object_class,
                                     PROP_IMG_FOCUS_POINT,
                                     g_param_spec_boolean(PROP_NAME_IMG_FOCUS_POINT,
@@ -803,8 +834,6 @@ static void entangle_preferences_class_init(EntanglePreferencesClass *klass)
                                                         G_PARAM_STATIC_NAME |
                                                         G_PARAM_STATIC_NICK |
                                                         G_PARAM_STATIC_BLURB));
-
-    g_type_class_add_private(klass, sizeof(EntanglePreferencesPrivate));
 }
 
 
@@ -814,7 +843,7 @@ EntanglePreferences *entangle_preferences_new(void)
 }
 
 
-static void entangle_preferences_ensure_data_dir(void)
+void entangle_preferences_ensure_gsettings_dir(void)
 {
     const char *const *dirs;
     const char *dir;
@@ -836,6 +865,33 @@ static void entangle_preferences_ensure_data_dir(void)
     setenv("GSETTINGS_SCHEMA_DIR", DATADIR "/glib-2.0/schemas", 1);
 }
 
+
+void entangle_preferences_ensure_girepository_dir(void)
+{
+#define GI_DIR LIBDIR "/girepository-1.0"
+    const char *path;
+
+    if (g_str_equal(PREFIX, "/usr")) {
+        ENTANGLE_DEBUG("Installed into /usr, standard girepository location");
+        return;
+    }
+
+    if ((path = getenv("GI_TYPELIB_PATH")) &&
+        strstr(path, GI_DIR)) {
+        ENTANGLE_DEBUG("Found dir " GI_DIR " in GI_TYPELIB_PATH");
+        return;
+    }
+
+    ENTANGLE_DEBUG("Setting %s in GSETTINGS_SCHEMA_DIR", DATADIR "/glib-2.0/schemas");
+    if (path) {
+        char *dir = g_strdup_printf("%s:%s", path, GI_DIR);
+        setenv("GI_TYPELIB_PATH", dir, 1);
+        g_free(dir);
+    } else {
+        setenv("GI_TYPELIB_PATH", GI_DIR, 1);
+    }
+}
+
 static void entangle_preferences_init(EntanglePreferences *picker)
 {
     EntanglePreferencesPrivate *priv;
@@ -843,8 +899,6 @@ static void entangle_preferences_init(EntanglePreferences *picker)
 
     priv = picker->priv = ENTANGLE_PREFERENCES_GET_PRIVATE(picker);
 
-    entangle_preferences_ensure_data_dir();
-
     settings = g_settings_new("org.entangle-photo.manager");
     priv->interfaceSettings = g_settings_get_child(settings,
                                                    SETTING_INTERFACE);
@@ -1325,9 +1379,10 @@ EntangleColourProfile *entangle_preferences_cms_get_rgb_profile(EntanglePreferen
 
     EntanglePreferencesPrivate *priv = prefs->priv;
     EntangleColourProfile *prof;
-
-    prof = entangle_colour_profile_new_file(g_settings_get_string(priv->cmsSettings,
-                                                                  SETTING_CMS_RGB_PROFILE));
+    gchar *val = g_settings_get_string(priv->cmsSettings,
+                                       SETTING_CMS_RGB_PROFILE);
+    prof = entangle_colour_profile_new_file(val);
+    g_free(val);
 
     return prof;
 }
@@ -1368,10 +1423,10 @@ EntangleColourProfile *entangle_preferences_cms_get_monitor_profile(EntanglePref
 
     EntanglePreferencesPrivate *priv = prefs->priv;
     EntangleColourProfile *prof;
-
-    prof = entangle_colour_profile_new_file(g_settings_get_string(priv->cmsSettings,
-                                                                  SETTING_CMS_MONITOR_PROFILE));
-
+    gchar *val = g_settings_get_string(priv->cmsSettings,
+                                       SETTING_CMS_MONITOR_PROFILE);
+    prof = entangle_colour_profile_new_file(val);
+    g_free(val);
     return prof;
 }
 
@@ -1637,6 +1692,46 @@ void entangle_preferences_img_set_mask_enabled(EntanglePreferences *prefs, gbool
 }
 
 
+/**
+ * entangle_preferences_img_get_overexposure_highlighting:
+ * @prefs: (transfer none): the preferences store
+ *
+ * Determine if areas of overexposure should be highlighted on
+ * the image display
+ *
+ * Returns: TRUE if overexposure should be highlighted
+ */
+gboolean entangle_preferences_img_get_overexposure_highlighting(EntanglePreferences *prefs)
+{
+    g_return_val_if_fail(ENTANGLE_IS_PREFERENCES(prefs), FALSE);
+
+    EntanglePreferencesPrivate *priv = prefs->priv;
+
+    return g_settings_get_boolean(priv->imgSettings,
+                                  SETTING_IMG_OVEREXPOSURE_HIGHLIGHTING);
+}
+
+
+/**
+ * entangle_preferences_img_set_overexposure_highlighting:
+ * @prefs: (transfer none): the preferences store
+ * @enabled: TRUE to highlight overeposure
+ *
+ * If @enabled is TRUE areas of overexposure will be highlighted on
+ * the image display
+ */
+void entangle_preferences_img_set_overexposure_highlighting(EntanglePreferences *prefs, gboolean enabled)
+{
+    g_return_if_fail(ENTANGLE_IS_PREFERENCES(prefs));
+
+    EntanglePreferencesPrivate *priv = prefs->priv;
+
+    g_settings_set_boolean(priv->imgSettings,
+                           SETTING_IMG_OVEREXPOSURE_HIGHLIGHTING, enabled);
+    g_object_notify(G_OBJECT(prefs), PROP_NAME_IMG_OVEREXPOSURE_HIGHLIGHTING);
+}
+
+
 /**
  * entangle_preferences_img_get_focus_point:
  * @prefs: (transfer none): the preferences store
