@@ -1,7 +1,7 @@
 /*
  *  Entangle: Tethered Camera Control & Capture
  *
- *  Copyright (C) 2009-2017 Daniel P. Berrange
+ *  Copyright (C) 2009-2018 Daniel P. Berrange
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -87,7 +87,9 @@ static void
 entangle_session_browser_scroll_to_selected(EntangleSessionBrowser *browser);
 
 G_DEFINE_TYPE_WITH_CODE(EntangleSessionBrowser, entangle_session_browser, GTK_TYPE_DRAWING_AREA,
-                        G_IMPLEMENT_INTERFACE(GTK_TYPE_SCROLLABLE, NULL))
+                        G_IMPLEMENT_INTERFACE(GTK_TYPE_SCROLLABLE, NULL)
+                        G_ADD_PRIVATE(EntangleSessionBrowser)
+                        )
 
 enum {
     PROP_O,
@@ -662,21 +664,25 @@ entangle_session_browser_motion_notify(GtkWidget *widget,
     EntangleSessionBrowser *browser = ENTANGLE_SESSION_BROWSER(widget);
     EntangleSessionBrowserPrivate *priv = browser->priv;
     GdkPixbuf *pixbuf = NULL;
+    gchar *demo = g_strdup("demo");
     GtkTargetEntry tentry[] = {
-        { g_strdup("demo"), GTK_TARGET_SAME_APP, 0,}
+        { demo, GTK_TARGET_SAME_APP, 0,}
     };
     GtkTargetList *tlist;
     GdkDragContext *ctx;
     EntangleMedia *media;
+    gboolean ret = FALSE;
 
     if (priv->dnd_start_y == -1 || priv->dnd_start_x == -1)
-        return FALSE;
+        goto cleanup;
 
     if (!gtk_drag_check_threshold(widget,
                                   priv->dnd_start_x,
                                   priv->dnd_start_y,
-                                  event->x, event->y))
-        return TRUE;
+                                  event->x, event->y)) {
+        ret = TRUE;
+        goto cleanup;
+    }
 
     media = entangle_session_browser_get_media_at_coords(browser,
                                                          priv->dnd_start_x,
@@ -684,7 +690,7 @@ entangle_session_browser_motion_notify(GtkWidget *widget,
 
     if (!media) {
         priv->dnd_start_x = priv->dnd_start_y = -1;
-        return FALSE;
+        goto cleanup;
     }
 
     if (ENTANGLE_IS_IMAGE(media)) {
@@ -693,7 +699,7 @@ entangle_session_browser_motion_notify(GtkWidget *widget,
 
         if (!pixbuf) {
             priv->dnd_start_x = priv->dnd_start_y = -1;
-            return FALSE;
+            goto cleanup;
         }
     }
 
@@ -712,7 +718,10 @@ entangle_session_browser_motion_notify(GtkWidget *widget,
 
     gtk_target_list_unref(tlist);
 
-    return TRUE;
+    ret = TRUE;
+  cleanup:
+    g_free(demo);
+    return ret;
 }
 
 static gboolean
@@ -886,8 +895,6 @@ static void entangle_session_browser_class_init(EntangleSessionBrowserClass *kla
                      0, NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
-
-    g_type_class_add_private(klass, sizeof(EntangleSessionBrowserPrivate));
 }
 
 
